from pydantic import BaseModel, Field
from datetime import datetime
from enum import StrEnum


class LogLevel(StrEnum):
    DEBUG = "debug"
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"
    CRITICAL = "critical"


class LogMessage(BaseModel):
    level: LogLevel
    message: str
    timestamp: datetime = Field(
        init=False,
        default_factory=datetime.now,
        description="The log message timestamp, defaults to the current time",
    )



from enum import StrEnum
from pydantic import BaseModel, Field


class ExceptionType(StrEnum):
    UNKNOWN = "unknown"
    BOT_IS_NOT_READY = "bot_is_not_ready"
    UNKNOWN_MESSAGE = "unknown_message"

    # Google login errors
    EMAIL_FROM_ENV_AND_FROM_MESSAGE_DOES_NOT_MATCH = (
        "email_from_env_and_from_message_does_not_match"
    )
    ## Elements not found
    EMAIL_FIELD_NOT_FOUND = "email_field_not_found"
    NEXT_BUTTON_NOT_FOUND = "next_button_not_found"
    PASSWORD_FIELD_NOT_FOUND = "password_field_not_found"
    SUBMIT_BUTTON_NOT_FOUND = "submit_button_not_found"
    LOGIN_FAILED = "login_failed"

    # Join meet errors
    CANNOT_LOAD_MEET_PAGE = "cannot_load_meet_page"
    JOIN_BUTTON_NOT_FOUND = "join_button_not_found"
    JOIN_TIMEOUT = "join_timeout"
    FAILED_JOIN_MEET = "failed_join_meet"

    # Start recording error
    BOT_IS_NOT_CONNECTED_MEET = "bot_is_not_connected_meet"
    RECORDING_ALREADY_STARTED = "recording_already_started"

    # Stop recording errors
    RECORDING_NOT_STARTED = "recording_not_started"

    # Publish recording errors
    RECORDING_NOT_FOUND = "recording_not_found"

    # Leave meet errors
    BOT_IS_NOT_IN_MEET = "bot_is_not_in_meet"


class ErrorMessage(BaseModel):
    exception_type: ExceptionType = Field(
        ...,
        description="Type of the exception that occurred",
    )
    message: str | None = Field(
        default=None,
        description="Detailed message describing the error",
    )



from typing import Annotated
from pydantic import Field, BeforeValidator
from .base import CommandType, BaseCommand


class JoinMeet(BaseCommand):
    meeting_id: str = Field(
        ..., examples=["abc-defg-hij"], description="The ID of the meeting to join"
    )
    type: Annotated[CommandType, BeforeValidator(lambda _: CommandType.JOIN_MEET)] = (
        Field(init=False, default=CommandType.JOIN_MEET, frozen=True)
    )



from typing_extensions import Annotated
from pydantic import BaseModel, Field, AfterValidator
from .type import CommandType


class BaseCommand(BaseModel):
    type: Annotated[
        CommandType,
        Field(
            init=False, description="This field must be overridden by all subclasses"
        ),
        AfterValidator(
            lambda v: v
            if v is not None
            else ValueError("Field 'type' must be overridden in subclasses")
        ),
    ]



from enum import StrEnum
from pydantic import BaseModel, Field
from datetime import datetime


class BotState(StrEnum):
    INITIALIZING = "init"
    READY = "ready"

    LOGGED_GOOGLE = "logged_google"
    WAITING_INVITATION_CONFIRMATION = "waiting_invitation_confirmation"
    CONNECTED_MEET = "connected_meet"

    RECORDING_STARTED = "recording_started"
    RECORDING_STOPPED = "recording_stopped"
    RECORDING_PUBLISHED = "recording_published"

    DEAD = "dead"

    @property
    def code(self) -> int:
        return list(self.__class__).index(self)

    @property
    def label(self) -> str:
        return self.value

    def __str__(self):
        return self.label

    @classmethod
    def from_code(cls, code: int) -> "BotState":
        try:
            return BotState(list(cls)[code])
        except IndexError:
            raise ValueError(f"Invalid status code: {code}")


class StateMessage(BaseModel):
    state: BotState = Field(..., description="The current state of the bot")
    timestamp: datetime = Field(
        default_factory=datetime.now,
        description="The timestamp when the state was updated",
    )



from pydantic import Field, BeforeValidator
from typing import Annotated
from .base import CommandType, BaseCommand


class LoginGoogleAccount(BaseCommand):
    email: str = Field(
        examples=["user@example.com"],
        description="The email of the Google account, needs to check if value from env is correct",
    )
    type: Annotated[
        CommandType, BeforeValidator(lambda _: CommandType.LOGIN_GOOGLE_ACCOUNT)
    ] = Field(init=False, default=CommandType.LOGIN_GOOGLE_ACCOUNT, frozen=True)
